#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import unicode_literals

""" Test the command line interface """

import pytest

from ..cli import _is_int,  _is_float, _resolve_type


@pytest.mark.parametrize("test_input,expected", [
    ("True", True),
    ("False", False),
    ("None", None),
    ("12.51", 12.51),
    ("10", 10),
    ("hello world", "hello world"),
    ("😍", "😍"),
])
def test_resolve_type(test_input, expected):
    assert _resolve_type(test_input) == expected


@pytest.mark.parametrize("value,expected", [
    (13.71, True),
    ("False", False),
    ("None", False),
    (-8.2, True),
    (10, True),
    ("10", True),
    ("12.31", True),
    ("hello world", False),
    ("😍", False),
])
def test_is_float(value, expected):
    assert (_is_float(value)) == expected


@pytest.mark.parametrize("value,expected", [
    (13.71, True),
    ("False", False),
    ("None", False),
    (-8.2, True),
    ("-23.2", False),
    (10, True),
    ("13", True),
    ("hello world", False),
    ("😍", False),
])
def test_is_int(value, expected):
    assert (_is_int(value)) == expected



import io
import os
import pytest
import shutil
import sys
import tempfile
import unittest

import nbformat

from ..api import read_notebook
from ..execute import execute_notebook, log_outputs
from ..exceptions import PapermillExecutionError
from . import get_notebook_path, RedirectOutput

python_2 = sys.version_info[0] == 2

class TestNotebookHelpers(unittest.TestCase):
    def setUp(self):
        self.test_dir = tempfile.mkdtemp()
        self.notebook_name = 'simple_execute_2.ipynb' if python_2 else 'simple_execute.ipynb'
        self.notebook_path = get_notebook_path(self.notebook_name)
        self.nb_test_executed_fname = os.path.join(self.test_dir, 'output_{}'.format(self.notebook_name))

    def tearDown(self):
        shutil.rmtree(self.test_dir)

    def test_cell_insertion(self):
        execute_notebook(self.notebook_path, self.nb_test_executed_fname, {'msg': 'Hello'})
        test_nb = read_notebook(self.nb_test_executed_fname)
        self.assertEqual(test_nb.node.cells[1].get('source'), u'# Parameters\nmsg = "Hello"\n')
        self.assertEqual(test_nb.parameters, {'msg': 'Hello'})

    def test_no_tags(self):
        notebook_name = 'no_parameters.ipynb'
        nb_test_executed_fname = os.path.join(self.test_dir, 'output_{}'.format(notebook_name))
        execute_notebook(get_notebook_path(notebook_name), nb_test_executed_fname, {'msg': 'Hello'})
        test_nb = read_notebook(nb_test_executed_fname)
        self.assertEqual(test_nb.node.cells[0].get('source'), u'# Parameters\nmsg = "Hello"\n')
        self.assertEqual(test_nb.parameters, {'msg': 'Hello'})

    def test_quoted_params(self):
        execute_notebook(self.notebook_path, self.nb_test_executed_fname, {'msg': '"Hello"'})
        test_nb = read_notebook(self.nb_test_executed_fname)
        self.assertEqual(test_nb.node.cells[1].get('source'), u'# Parameters\nmsg = "\\"Hello\\""\n')
        self.assertEqual(test_nb.parameters, {'msg': '"Hello"'})


class TestBrokenNotebook1(unittest.TestCase):

    def setUp(self):
        self.test_dir = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.test_dir)

    def test(self):
        path = get_notebook_path('broken1.ipynb')
        result_path = os.path.join(self.test_dir, 'broken1.ipynb')
        with self.assertRaises(PapermillExecutionError):
            execute_notebook(path, result_path)
        nb = read_notebook(result_path)
        self.assertEqual(nb.node.cells[0].cell_type, "markdown")
        self.assertEqual(nb.node.cells[1].execution_count, 1)
        self.assertEqual(nb.node.cells[2].execution_count, 2)
        self.assertEqual(nb.node.cells[2].outputs[0].output_type, 'error')
        self.assertEqual(nb.node.cells[3].execution_count, None)


class TestBrokenNotebook2(unittest.TestCase):

    def setUp(self):
        self.test_dir = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.test_dir)

    def test(self):
        path = get_notebook_path('broken2.ipynb')
        result_path = os.path.join(self.test_dir, 'broken2.ipynb')
        with self.assertRaises(PapermillExecutionError):
            execute_notebook(path, result_path)
        nb = read_notebook(result_path)
        self.assertEqual(nb.node.cells[0].cell_type, "markdown")
        self.assertEqual(nb.node.cells[1].execution_count, 1)
        self.assertEqual(nb.node.cells[2].execution_count, 2)
        self.assertEqual(nb.node.cells[2].outputs[0].output_type, 'display_data')
        self.assertEqual(nb.node.cells[2].outputs[1].output_type, 'error')
        self.assertEqual(nb.node.cells[3].execution_count, None)


class TestLogging(unittest.TestCase):

    def setUp(self):
        self.saved_stdout = sys.stdout
        self.out = io.StringIO()
        sys.stdout = self.out

    def tearDown(self):
        sys.stdout = self.saved_stdout

    def test_logging(self):

        nb = nbformat.read(get_notebook_path('test_logging.ipynb'), as_version=4)

        # stderr output
        with RedirectOutput() as redirect:
            log_outputs(nb.cells[0])
            stdout = redirect.get_stdout()
            self.assertEqual(stdout, u'Out [1] --------------------------------\n\n')
            stderr = redirect.get_stderr()
            self.assertEqual(stderr, u'Out [1] --------------------------------\nINFO:test:test text\n\n')

        # stream output
        with RedirectOutput() as redirect:
            log_outputs(nb.cells[1])
            stdout = redirect.get_stdout()
            self.assertEqual(stdout, u'Out [2] --------------------------------\nhello world\n\n')
            stderr = redirect.get_stderr()
            self.assertEqual(stderr, u'Out [2] --------------------------------\n\n')

        # text/plain output
        with RedirectOutput() as redirect:
            log_outputs(nb.cells[2])
            stdout = redirect.get_stdout()
            self.assertEqual(stdout,
                             (
                                "Out [3] --------------------------------\n"
                                "<matplotlib.axes._subplots.AxesSubplot at 0x7f8391f10290>\n"
                                "<matplotlib.figure.Figure at 0x7f830af7b350>\n"
                                )
                             )
            stderr = redirect.get_stderr()
            self.assertEqual(stderr, u'Out [3] --------------------------------\n\n')



# -*- coding: utf-8 -*-
"""Main `papermill` interface."""

from __future__ import unicode_literals
import base64

import click
import yaml

from .execute import execute_notebook
from .iorw import read_yaml_file


@click.command(context_settings=dict(help_option_names=['-h', '--help']))
@click.argument('notebook_path')
@click.argument('output_path')
@click.option(
    '--parameters',
    '-p',
    help='Parameters to pass to the parameters cell.',
    multiple=True,
    nargs=2)
@click.option(
    '--parameters_raw',
    '-r',
    help='Parameters to be read as raw string.',
    multiple=True,
    nargs=2)
@click.option(
    '--parameters_file', '-f', help='Path to YAML file containing parameters.')
@click.option(
    '--parameters_yaml', '-y', help='YAML string to be used as parameters.')
@click.option(
    '--parameters_base64',
    '-b',
    help='Base64 encoded YAML string as parameters.')
@click.option('--kernel', '-k', help='Name of kernel to run.')
@click.option(
    '--progress-bar/--no-progress-bar',
    default=True,
    help="Flag for turning on the progress bar.")
@click.option(
    '--log-output/--no-log-output',
    default=False,
    help="Flag for writing notebook output to stderr.")
def papermill(notebook_path, output_path, parameters, parameters_raw,
              parameters_file, parameters_yaml, parameters_base64, kernel,
              progress_bar, log_output):
    """Utility for executing a single notebook on a container.

    Take a source notebook, apply parameters to the source notebook,
    execute the notebook with the kernel specified, and save the
    output in the destination notebook.

    """
    if parameters_base64:
        parameters_final = yaml.load(base64.b64decode(parameters_base64))
    elif parameters_yaml:
        parameters_final = yaml.load(parameters_yaml)
    elif parameters_file:
        parameters_final = read_yaml_file(parameters_file)
    else:
        # Read in Parameters
        parameters_final = {}
        for name, value in parameters:
            parameters_final[name] = _resolve_type(value)
        for name, value in parameters_raw:
            parameters_final[name] = value

    execute_notebook(
        notebook_path,
        output_path,
        parameters_final,
        kernel_name=kernel,
        progress_bar=progress_bar,
        log_output=log_output)


def _resolve_type(value):
    if value == "True":
        return True
    elif value == "False":
        return False
    elif value == "None":
        return None
    elif _is_int(value):
        return int(value)
    elif _is_float(value):
        return float(value)
    else:
        return value


def _is_int(value):
    """
    Casting is used to check if the value
    in the cli can be converted to an int
    """
    try:
        int(value)
    except ValueError:
        return False
    else:
        return True


def _is_float(value):
    """
    Casting is used to check if the value
    in the cli can be converted to a float
    """
    try:
        float(value)
    except ValueError:
        return False
    else:
        return True


if __name__ == '__main__':
    papermill()
